package de.hdm_stuttgart.mi.bb;

import de.hdm_stuttgart.mi.bb.model.Topic;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.jackson.JacksonFeature;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

public class Id2Topic {

    private static Logger log = LogManager.getLogger(Id2Topic.class);
    static private final WebTarget topicsTarget ;
    static {
        // Disable startup info
        final java.util.logging.Logger mongoLogger = java.util.logging.Logger.getLogger( "org.mongodb.driver" );
        mongoLogger.setLevel(java.util.logging.Level.SEVERE); // e.g. or Log.WARNING, etc.

        final String
            configPathName= System.getProperty("user.home"),
            configFileBaseName = "bbtopics",
            configFileFullyQualifiedName = configPathName + '/' + configFileBaseName + ".properties";

        final File file = new File(configPathName);

        ResourceBundle tmpConfig = null;
        try {
            final URL[] urls = new URL[]{file.toURI().toURL()};
            final ClassLoader loader = new URLClassLoader(urls);
            tmpConfig = ResourceBundle.getBundle(configFileBaseName, Locale.getDefault(), loader);
        } catch (final MalformedURLException | MissingResourceException e) {
            log.fatal("Unable to open URI " + configFileFullyQualifiedName, e);
            System.exit(1);
        }
        config = tmpConfig;

        final Client client = ClientBuilder.newClient(new ClientConfig().register(new JacksonFeature()));
        topicsTarget  = client.target(tmpConfig.getString("nodebbWriteApiTopicsEndpoint"));
    }

    static private final ResourceBundle config;

    static private final
    ModifyNodebbTopic db = new ModifyNodebbTopic(
            config.getString("mongoUri"),
            config.getString("nodebbDb"));

    static private final String bearerToken = config.getString("nodebbBearerToken");

    static private final String
            KEY_PAYLOAD = "payload",
            KEY_TOPICDATA = "topicData";

    public static void main(String[] args) {

        final SAXBuilder saxBuilder = new SAXBuilder();

        if (0 == args.length) {
            System.err.println("Usage: id2topic <docbook2topic basefilename>");
        } else {
            final File inputFile = new File(args[0] + ".xml");

            try {
                final Document document = saxBuilder.build(inputFile);
                handleDocbok2idMap(document.getRootElement());

                final Format outFormat = Format.getPrettyFormat();

                // Serialize to console
                final XMLOutputter printer = new XMLOutputter(outFormat);

                final String outputFileName = args[0] + ".tid.xml";
                final FileWriter outputFile = new FileWriter(outputFileName);
                printer.output(document.getRootElement(), outputFile);
            } catch (JDOMException e) {
                System.err.println("Unable to parse " + inputFile);
            } catch (IOException e) {
                System.err.println("Unable to read " + inputFile);
            }
        }
    }

    static private void handleDocbok2idMap(final Element map) {
        final String mapPrefix = map.getAttributeValue("prefix");

        for (final Element entry : map.getChildren("entry")) {

            final int tid = createTopic(
                    mapPrefix + ":" + entry.getAttributeValue("id"),
                    Integer.parseInt(entry.getAttributeValue("forumId")),
                    entry.getChildText("title"),
                    entry.getChildText("content"));

            entry.setAttribute("tid", Integer.toString(tid));
        }
    }

    static private int createTopic(final String hdmId, final int categoryId, final String title, final String content) {

        final int tid = db.hdmIdExists(hdmId);
        if (tid < 0) { // No related topic
            final Topic topic = new Topic(categoryId, title, content);
            final Entity jsonTopic = Entity.json(topic);
            final Map<String, Object> response = (Map<String, Object>)
                topicsTarget.request(MediaType.APPLICATION_JSON).
                header(HttpHeaders.AUTHORIZATION, "Bearer " + bearerToken).
                accept(MediaType.TEXT_PLAIN_TYPE).
                post(jsonTopic, Map.class);

            final Map<String, Object> topicData = (Map<String, Object>)
                ((Map<String, Object>)(response.get(KEY_PAYLOAD))).
                get(KEY_TOPICDATA);

            final

            int newTid = (int) topicData.get("tid");
            db.addHdmId(newTid, hdmId);
            log.info("Created topic " + topic + " (hdmId = " + hdmId + ", tid = " + newTid + ")");
            return newTid;
        } else {
            log.info ("Found topic (hdmId = " +  hdmId + ", tid = " + tid + ")");
            return tid;
            // Possibly aligning title and content with lecture notes?
            // Difficult: Incomplete NodeBB write API --> required MongoDB fiddling

            /*
            final WebTarget hdmTtopicTarget =
                    client.target(config.getString("nodebbReadApiTopicsEndpoint") + "/" + tid);

            final Response response = hdmTtopicTarget.request(MediaType.APPLICATION_JSON_TYPE).get();

            final Map<String, Object> hdmTopicMap = response.readEntity(Map.class);
            log.info("-----------------------Topic [tid=" + tid + "] data:");
            for (String k : hdmTopicMap) {
                System.out.println("key '" + k + "', val '" + m.get(k) + "'");
            }
             */
        }
    }
}
